SystemOrganization addCategory: #MonticelloConfigurations!

MCTool subclass: #MCConfigurationBrowser
	instanceVariableNames: 'configuration dependencyIndex repositoryIndex'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCConfigurationBrowser commentStamp: 'dtl 5/10/2010 21:48' prior: 0!
A MCConfigurationBrowser displays an MCConfiguration, and edits the configuration to add or remove package dependencies and repository specifications. It allows a configuration to be stored in a repository or posted to an update stream.!

!MCConfigurationBrowser class methodsFor: 'class initialization' stamp: 'bf 3/21/2005 19:46'!
initialize
	TheWorldMenu registerOpenCommand: { 'Monticello Configurations' . { self . #open }. 'Monticello Configuration Browser' }.! !

!MCConfigurationBrowser class methodsFor: 'opening' stamp: 'bf 3/21/2005 19:50'!
open
	^self new show! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 3/23/2005 22:08'!
add
	(self pickWorkingCopiesSatisfying: [:each | (self includesPackage: each package) not])
		do: [:wc |
			wc ancestors isEmpty
				ifTrue: [self inform: 'You must save ', wc packageName, ' first!!
Skipping this package']
				ifFalse: [
					self dependencies add: (MCVersionDependency
						package: wc package
						info: wc ancestors first)]].
	self changed: #dependencyList; changed: #description! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 3/23/2005 22:08'!
addDependency
	(self pickWorkingCopiesSatisfying: [:each | (self includesPackage: each package) not])
		do: [:wc |
			wc ancestors isEmpty
				ifTrue: [self inform: 'You must save ', wc packageName, ' first!!
Skipping this package']
				ifFalse: [
					self dependencies add: (MCVersionDependency
						package: wc package
						info: wc ancestors first)]].
	self changed: #dependencyList; changed: #description! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'ul 12/12/2009 14:10'!
addRepository
	(self pickRepositorySatisfying: [:ea | (self repositories includes: ea) not])
		ifNotNil: [:repo |
			(repo isKindOf: MCHttpRepository)
				ifFalse: [^self inform: 'Only HTTP repositories are supported'].
			self repositories add: repo.
			self changed: #repositoryList.
		]! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'bf 6/22/2005 12:26'!
buttonSpecs
	^ #(('Add' add 'Add a dependency')
		('Update' updateMenu 'Update dependencies')
		('Install' installMenu 'Load/Merge/Upgrade into image')
		('Up' up 'Move item up in list' canMoveUp)
		('Down' down 'Move item down in list' canMoveDown)
		('Remove' remove 'Remove item' canRemove)
		('Store' store 'store configuration')
		('Post' post 'Post this configuration to an update stream')
		)! !

!MCConfigurationBrowser methodsFor: 'testing' stamp: 'bf 3/23/2005 20:44'!
canMoveDown
	^self index between: 1 and: self maxIndex - 1 ! !

!MCConfigurationBrowser methodsFor: 'testing' stamp: 'bf 3/23/2005 20:44'!
canMoveUp
	^self index > 1! !

!MCConfigurationBrowser methodsFor: 'testing' stamp: 'bf 3/23/2005 20:45'!
canRemove
	^self index > 0! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 5/27/2005 19:54'!
changedButtons
	self changed: #canMoveDown.
	self changed: #canMoveUp.
	self changed: #canRemove.! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 20:55'!
changedList
	self dependencyIndex > 0 ifTrue: [^self changed: #dependencyList].
	self repositoryIndex > 0 ifTrue: [^self changed: #repositoryList].
	self error: 'nothing selected'! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 4/19/2005 17:36'!
checkDependencies
	^self checkModified and: [self checkMissing]! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 4/19/2005 17:35'!
checkMissing
	| missing |
	missing := (self dependencies collect: [:ea | ea versionInfo name]) asSet.

	self repositories
		do: [:repo |
			repo allVersionNames
				do: [:found | missing remove: found ifAbsent: []]]
		displayingProgress: 'searching versions'.

	^missing isEmpty or: [
		self selectDependency: missing anyOne.
		self confirm: (String streamContents: [:strm |
			strm nextPutAll: 'No repository found for'; cr.
			missing do: [:r | strm nextPutAll: r; cr].
			strm nextPutAll: 'Do you still want to store?'])]
	! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 4/19/2005 17:37'!
checkModified
	| modified |
	modified := self dependencies select: [:dep |
		dep isFulfilled and: [dep package workingCopy modified]].
	
	^modified isEmpty or: [
		self selectDependency: modified anyOne.
		self confirm: (String streamContents: [:strm |
			strm nextPutAll: 'These packages are modified:'; cr.
			modified do: [:dep | strm nextPutAll: dep package name; cr].
			strm nextPutAll: 'Do you still want to store?'])]
	! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'bf 3/24/2005 00:45'!
checkRepositories
	| bad |
	bad := self repositories reject: [:repo | repo isKindOf: MCHttpRepository].
	^bad isEmpty or: [
		self selectRepository: bad first.
		self inform: (String streamContents: [:strm |
			strm nextPutAll: 'Please remove these repositories:'; cr.
			bad do: [:r | strm nextPutAll: r description; cr].
			strm nextPutAll: '(only HTTP repositories are supported)']).
		false].
! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'bf 3/24/2005 00:47'!
checkRepositoryTemplates
	"unused for now - we only do HTTP"
	| bad |
	bad := self repositories select: [:repo | repo creationTemplate isNil].
	^bad isEmpty or: [
		self selectRepository: bad first.
		self inform: (String streamContents: [:strm |
			strm nextPutAll: 'Creation template missing for'; cr.
			bad do: [:r | strm nextPutAll: r description; cr].
			strm nextPutAll: 'Please fill in the details first!!']).
		false].
! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 3/21/2005 16:03'!
configuration
	^configuration ifNil: [configuration := MCConfiguration new]! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 3/21/2005 14:56'!
configuration: aConfiguration
	configuration := aConfiguration! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'bf 4/19/2005 16:51'!
defaultExtent
	^ 350@500! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 3/21/2005 16:35'!
dependencies
	^self configuration dependencies
! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 4/19/2005 16:02'!
dependencies: aCollection
	self configuration dependencies: aCollection.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 17:56'!
dependencyIndex
	^dependencyIndex ifNil: [0]! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 5/27/2005 19:55'!
dependencyIndex: anInteger
	dependencyIndex := anInteger.
	dependencyIndex > 0
		ifTrue: [self repositoryIndex: 0].
	self changed: #dependencyIndex; changed: #description.
	self changedButtons.! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 4/19/2005 17:28'!
dependencyList
	^self dependencies collect: [:dep | 
		Text string: dep versionInfo name
			attributes: (Array streamContents: [:attr |
				dep isFulfilledByAncestors
					ifFalse: [attr nextPut: TextEmphasis bold]
					ifTrue: [dep isCurrent ifFalse: [attr nextPut: TextEmphasis italic]].
			])]
! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'dtl 5/10/2010 21:11'!
dependencyMenu: aMenu
	self fillMenu: aMenu fromSpecs: #(('add dependency...' addDependency)).
	self selectedDependency ifNotNil: [
		self fillMenu: aMenu fromSpecs: #(('remove dependency' remove))].
	^aMenu
! !

!MCConfigurationBrowser methodsFor: 'description' stamp: 'ul 12/12/2009 14:10'!
description
	self selectedDependency ifNotNil: [:dep | ^ ('Package: ', dep package name, String cr,
		dep versionInfo summary) asText].
	self selectedRepository ifNotNil: [:repo | ^repo creationTemplate
		ifNotNil: [repo creationTemplate asText]
		ifNil: [repo asCreationTemplate asText addAttribute: TextColor red]].
	^ ''
! !

!MCConfigurationBrowser methodsFor: 'description' stamp: 'ul 12/12/2009 14:10'!
description: aText

	self selectedRepository ifNotNil: [:repo | 
		| new | 
		new := MCRepository readFrom: aText asString.
		(new class = repo class 
			and: [new description = repo description])
				ifTrue: [
					repo creationTemplate: aText asString.
					self changed: #description]
				ifFalse: [
					self inform: 'This does not match the previous definition!!'
				]
	].

! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 3/23/2005 21:01'!
down
	self canMoveDown ifTrue: [
		self list swap: self index with: self index + 1.
		self index: self index + 1.
		self changedList.
	].
! !

!MCConfigurationBrowser methodsFor: 'testing' stamp: 'bf 3/21/2005 17:15'!
includesPackage: aPackage
	^self dependencies anySatisfy: [:each | each package = aPackage]! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 20:43'!
index
	^self dependencyIndex max: self repositoryIndex! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 21:00'!
index: anInteger
	self dependencyIndex > 0 ifTrue: [^self dependencyIndex: anInteger].
	self repositoryIndex > 0 ifTrue: [^self repositoryIndex: anInteger].
	anInteger > 0 ifTrue: [self error: 'cannot select']! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
installMenu

	| menu |
	menu := MenuMorph new defaultTarget: self.
	menu add: 'load packages' action: #load.
	menu add: 'merge packages' action: #merge.
	menu add: 'upgrade packages' action: #upgrade.
	menu popUpInWorld.! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 20:51'!
list
	self dependencyIndex > 0 ifTrue: [^self dependencies].
	self repositoryIndex > 0 ifTrue: [^self repositories].
	^#()! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 4/19/2005 17:42'!
load
	self configuration load.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
loadMenu

	| menu |
	menu := MenuMorph new defaultTarget: self.
	menu add: 'update from image' action: #updateFromImage.
	menu add: 'update from repositories' action: #updateFromRepositories.
	menu popUpInWorld.
! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 20:52'!
maxIndex
	^ self list size! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 4/19/2005 17:42'!
merge
	self configuration merge.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'ul 3/14/2011 17:21'!
pickName
	| name |
	name := UIManager default
		request: 'Name (.', self configuration writerClass extension, ' will be appended):'
		initialAnswer: self configuration suggestedNameOfNextVersion.
	^ name isEmpty ifFalse: [name]! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'bf 3/23/2005 21:11'!
pickRepository
	^self pickRepositorySatisfying: [:ea | true]
! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'ar 8/6/2009 18:23'!
pickRepositorySatisfying: aBlock
	| index list |
	list := MCRepositoryGroup default repositories select: aBlock.
	index := UIManager default chooseFrom: (list collect: [:ea | ea description])
		title: 'Repository:'.
	^ index = 0 ifFalse: [list at: index]! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'ar 8/6/2009 20:41'!
pickWorkingCopiesSatisfying: aBlock
	| copies item |
	copies := (MCWorkingCopy allManagers select: aBlock)
		asSortedCollection: [:a :b | a packageName <= b packageName].
	item := UIManager default chooseFrom: #('match ...'),(copies collect: [:ea | ea packageName]) lines: #(1) title: 'Package:'.
	item = 1 ifTrue: [
		| pattern |
		pattern := UIManager default request: 'Packages matching:' initialAnswer: '*'.
		^pattern isEmptyOrNil
			ifTrue: [#()]
			ifFalse: [
				(pattern includes: $*) ifFalse: [pattern := '*', pattern, '*'].
				copies select: [:ea | pattern match: ea packageName]]
	].
	^ item = 0
		ifTrue: [#()]
		ifFalse: [{copies at: item - 1}]! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'ul 3/14/2011 17:21'!
post
	"Take the current configuration and post an update"
	| name update managers names choice |
	(self checkRepositories and: [self checkDependencies]) ifFalse: [^self].
	name := UIManager default
		request: 'Update name (.cs) will be appended):'
		initialAnswer: self configuration suggestedNameOfNextVersion.
	name isEmpty ifTrue:[^self].
	self configuration name: name.
	update := MCPseudoFileStream on: (String new: 100).
	update localName: name, '.cs'.
	update nextPutAll: '"Change Set:		', name.
	update cr; nextPutAll: 'Date:			', Date today printString.
	update cr; nextPutAll: 'Author:			Posted by Monticello'.
	update cr; cr; nextPutAll: 'This is a configuration map created by Monticello."'.

	update cr; cr; nextPutAll: '(MCConfiguration fromArray: #'.
	self configuration fileOutOn: update.
	update nextPutAll: ') upgrade.'.
	update position: 0.

	managers := Smalltalk at: #UpdateManager ifPresent:[:mgr| mgr allRegisteredManagers].
	managers ifNil:[managers := #()].
	managers size > 0 ifTrue:[
		| servers index |
		servers := ServerDirectory groupNames asSortedArray.
		names := (managers collect:[:each| each packageVersion]), servers.
		index := UIManager default chooseFrom: names lines: {managers size}.
		index = 0 ifTrue:[^self].
		index <= managers size ifTrue:[
			| mgr |
			mgr := managers at: index.
			^mgr publishUpdate: update.
		].
		choice := names at: index.
	] ifFalse:[
		names := ServerDirectory groupNames asSortedArray.
		choice := UIManager default chooseFrom: names values: names.
		choice == nil ifTrue: [^ self].
	].
	(ServerDirectory serverInGroupNamed: choice) putUpdate: update.! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 3/23/2005 21:05'!
remove
	self canRemove ifTrue: [
		self list removeAt: self index.
		self changedList.
		self updateIndex.
	].
! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'dtl 5/10/2010 21:01'!
removeRepository
	repositoryIndex > 0
		ifTrue: [self repositories removeAt: repositoryIndex.
			repositoryIndex := 0.
			self changed: #repositoryList]! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 3/23/2005 17:41'!
repositories
	^ self configuration repositories! !

!MCConfigurationBrowser methodsFor: 'accessing' stamp: 'bf 3/23/2005 21:15'!
repositories: aCollection
	^self configuration repositories: aCollection
! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 17:57'!
repositoryIndex
	^repositoryIndex ifNil: [0]! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 5/27/2005 19:55'!
repositoryIndex: anInteger
	repositoryIndex := anInteger.
	repositoryIndex > 0
		ifTrue: [self dependencyIndex: 0].
	self changed: #repositoryIndex; changed: #description.
	self changedButtons.! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'bf 3/23/2005 21:15'!
repositoryList
	^self repositories collect: [:ea | ea description]
! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'dtl 5/10/2010 20:56'!
repositoryMenu: aMenu
	self fillMenu: aMenu fromSpecs: #(('add repository...' addRepository)).
	self selectedRepository ifNotNil: [
		self fillMenu: aMenu fromSpecs: #(('remove repository' removeRepository))].
	^aMenu
! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 23:16'!
selectDependency: aDependency
	self dependencyIndex: (self dependencies indexOf: aDependency)! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 23:15'!
selectRepository: aRepository
	self repositoryIndex: (self repositories indexOf: aRepository)! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'bf 3/23/2005 17:56'!
selectedDependency
	^ self dependencies at: self dependencyIndex ifAbsent: []! !

!MCConfigurationBrowser methodsFor: 'dependencies' stamp: 'ul 12/12/2009 14:10'!
selectedPackage
	^ self selectedDependency ifNotNil: [:dep | dep package]! !

!MCConfigurationBrowser methodsFor: 'repositories' stamp: 'bf 3/23/2005 17:58'!
selectedRepository
	^ self repositories at: self repositoryIndex ifAbsent: []! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'ul 12/12/2009 14:10'!
store
	(self checkRepositories and: [self checkDependencies]) ifFalse: [^self].
	self pickName ifNotNil: [:name |
		self configuration name: name.
		self pickRepository ifNotNil: [:repo |
			repo storeVersion: self configuration]].! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 3/23/2005 20:53'!
up
	self canMoveUp ifTrue: [
		self list swap: self index with: self index - 1.
		self index: self index - 1.
		self changedList.
	].! !

!MCConfigurationBrowser methodsFor: 'updating' stamp: 'bf 5/23/2005 17:44'!
updateFromImage
	self configuration updateFromImage.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'updating' stamp: 'bf 5/23/2005 17:44'!
updateFromRepositories
	self configuration updateFromRepositories.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'selection' stamp: 'bf 3/23/2005 21:00'!
updateIndex
	self index > 0 ifTrue: [self index: (self index min: self maxIndex)]! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'stephaneducasse 2/4/2006 20:47'!
updateMenu

	| menu |
	menu := MenuMorph new defaultTarget: self.
	menu add: 'update from image' action: #updateFromImage.
	menu add: 'update from repositories' action: #updateFromRepositories.
	menu popUpInWorld.! !

!MCConfigurationBrowser methodsFor: 'actions' stamp: 'bf 4/19/2005 17:43'!
upgrade
	self configuration upgrade.
	self changed: #dependencyList; changed: #description
! !

!MCConfigurationBrowser methodsFor: 'morphic ui' stamp: 'bf 3/23/2005 22:01'!
widgetSpecs
	^ #(
		((buttonRow) (0 0 1 0) (0 0 0 30))
		((listMorph:selection:menu: dependencyList dependencyIndex dependencyMenu:) (0 0 1 1) (0 30 0 -180))
		((listMorph:selection:menu: repositoryList repositoryIndex repositoryMenu:) (0 1 1 1) (0 -180 0 -120))
		((textMorph: description) (0 1 1 1) (0 -120 0 0))
	 	)! !

RWBinaryOrTextStream subclass: #MCPseudoFileStream
	instanceVariableNames: 'localName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCPseudoFileStream commentStamp: '<historical>' prior: 0!
A pseudo file stream which can be used for updates.!

!MCPseudoFileStream methodsFor: 'accessing' stamp: 'ar 4/14/2005 19:54'!
localName
	^localName! !

!MCPseudoFileStream methodsFor: 'accessing' stamp: 'ar 4/14/2005 19:54'!
localName: aString
	localName := aString! !

MCWriter subclass: #MCMcmWriter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCMcmWriter commentStamp: 'dtl 5/10/2010 22:20' prior: 0!
An MCMcmWriter stores an MCConfiguration on a stream in the form of an array specification.
!

!MCMcmWriter class methodsFor: 'writing' stamp: 'stephaneducasse 2/4/2006 20:47'!
fileOut: aConfiguration on: aStream
	| inst |
	inst := self on: aStream.
	inst writeConfiguration: aConfiguration.
	inst close.
	
! !

!MCMcmWriter class methodsFor: 'accessing' stamp: 'bf 3/22/2005 10:49'!
readerClass
	^ MCMcmReader! !

!MCMcmWriter methodsFor: 'writing' stamp: 'bf 3/22/2005 18:00'!
close
	stream close! !

!MCMcmWriter methodsFor: 'writing' stamp: 'bf 3/24/2005 01:50'!
writeConfiguration: aConfiguration

	stream nextPut: $(.

	aConfiguration repositories do: [:ea | 
		stream cr.
		stream nextPutAll: 'repository '.
		(MCConfiguration repositoryToArray: ea) printElementsOn: stream].

	aConfiguration dependencies do: [:ea | 
		stream cr.
		stream nextPutAll: 'dependency '.
		(MCConfiguration dependencyToArray: ea) printElementsOn: stream].

	stream cr.
	stream nextPut: $).
	stream cr.! !

Object subclass: #MCConfiguration
	instanceVariableNames: 'name dependencies repositories log'
	classVariableNames: 'DefaultLog ExtraProgressInfo LogToFile'
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCConfiguration commentStamp: 'dtl 5/10/2010 23:03' prior: 0!
An MCConfiguration specifies the configuration of a set of related Monticello packages. It maintains an ordered list of package versions and a list of repositories in which the packages may be found.

An MCConfiguration may be filed out for storage as an array specification, and new instances can be created from a stored array specification.
!

!MCConfiguration class methodsFor: 'accessing' stamp: 'ar 5/27/2005 17:27'!
defaultLog
	"Answer the default configuration log"
	^DefaultLog! !

!MCConfiguration class methodsFor: 'accessing' stamp: 'ar 5/27/2005 17:27'!
defaultLog: aStream
	"Set the default configuration log"
	DefaultLog := aStream.! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 3/24/2005 01:43'!
dependencyFromArray: anArray
	^MCVersionDependency
		package: (MCPackage named: anArray first)
		info: (
			MCVersionInfo
			name: anArray second
			id: (UUID fromString: anArray third)
			message: nil
			date: nil
			time: nil
			author: nil
			ancestors: nil)! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 3/24/2005 01:44'!
dependencyToArray: aDependency
	^ {
		aDependency package name . 
		aDependency versionInfo name . 
		aDependency versionInfo id asString }! !

!MCConfiguration class methodsFor: 'preferences' stamp: 'ar 12/30/2009 17:30'!
extraProgressInfo
	"Answer true for additional progress info during load. 
	With the newly added MC down/upload operations this seems unnecessary
	but some people might disagree, so let's leave it as a preference right now"
	<preference: 'Extra Progress Info' 
		category: 'Monticello' 
		description: 'If true, additional progress information is displayed when loading MC configurations (i.e., during updates)' 
		type: #Boolean>
	^ExtraProgressInfo ifNil:[false]! !

!MCConfiguration class methodsFor: 'preferences' stamp: 'ar 12/30/2009 17:30'!
extraProgressInfo: aBool
	"Whether to display for additional progress info during load."
	ExtraProgressInfo := aBool.
! !

!MCConfiguration class methodsFor: 'instance creation' stamp: 'bf 3/24/2005 01:51'!
fromArray: anArray
	| configuration |
	configuration := self new.
	anArray pairsDo: [:key :value |
		key = #repository
			ifTrue: [configuration repositories add: (self repositoryFromArray: value)].
		key = #dependency
			ifTrue: [configuration dependencies add: (self dependencyFromArray: value)].
	].
	^configuration! !

!MCConfiguration class methodsFor: 'class initialization' stamp: 'bf 4/20/2005 17:20'!
initialize
	"MCConfiguration initialize"

	Preferences addPreference: #upgradeIsMerge
		categories: #('updates') default: false 
		balloonHelp: 'When upgrading packages, use merge instead of load'.! !

!MCConfiguration class methodsFor: 'preferences' stamp: 'ar 8/21/2009 21:18'!
logToFile
	"Whether to log configuration info to files by default.
	If true, logs to a file named after the configuration (config.nn.log).
	If false, logs to the transcript."
	<preference: 'Log config info to disk' 
		category: 'Monticello' 
		description: 'If true, configuration information (such as change logs) are logged to disk instead of the Transcript. The log file is named after the configuration map (config.nn.log)' 
		type: #Boolean>
	^LogToFile ifNil:[true].! !

!MCConfiguration class methodsFor: 'preferences' stamp: 'ar 8/21/2009 21:13'!
logToFile: aBool
	"Whether to log configuration info to files by default.
		MCConfiguration logToFile: true.
		MCConfiguration logToFile: false.
	"
	LogToFile := aBool! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 6/9/2005 14:25'!
repositoryFromArray: anArray
	^ MCRepositoryGroup default repositories
		detect: [:repo | repo description = anArray first]
		ifNone: [
			MCHttpRepository
				location: anArray first
				user: ''
				password: '']! !

!MCConfiguration class methodsFor: 'converting' stamp: 'bf 3/24/2005 01:51'!
repositoryToArray: aRepository
	^ {aRepository description}! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 3/22/2005 22:09'!
browse
	(MCConfigurationBrowser new configuration: self) show! !

!MCConfiguration methodsFor: 'faking' stamp: 'bf 3/24/2005 01:19'!
changes
	^MCPatch operations: #()! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/21/2005 16:32'!
dependencies
	^dependencies ifNil: [dependencies := OrderedCollection new]! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/21/2005 18:40'!
dependencies: aCollection
	dependencies := aCollection! !

!MCConfiguration methodsFor: 'private' stamp: 'ar 12/30/2009 17:31'!
depsSatisfying: selectBlock versionDo: verBlock displayingProgress: progressString
	| repoMap count action |
	repoMap := Dictionary new.
	self repositories do: [:repo |
		MCRepositoryGroup default addRepository: repo.
		repo allVersionNames
			ifEmpty: [self logWarning: 'cannot read from ', repo description]
			ifNotEmptyDo: [:all | all do: [:ver | repoMap at: ver put: repo]]].

	count := 0.
	action := [:dep |
		| ver repo |
		ver := dep versionInfo name.
		repo := repoMap at: ver ifAbsent: [
			self logError: 'Version ', ver, ' not found in any repository'.
			self logError: 'Aborting'.
			^count].
		(selectBlock value: dep) ifTrue: [
			| new |
			new := self versionNamed: ver for: dep from: repo.
			new ifNil: [
					self logError: 'Could not download version ', ver, ' from ', repo description.
					self logError: 'Aborting'.
					^count]
				ifNotNil: [
					self logUpdate: dep package with: new.
					self class extraProgressInfo
						ifTrue:[ProgressNotification signal: '' extra: 'Installing ', ver].
					verBlock value: new.
					count := count + 1.
				]
		].
		dep package workingCopy repositoryGroup addRepository: repo.
	].
	self class extraProgressInfo 
		ifTrue:[self dependencies do: action displayingProgress: progressString]
		ifFalse:[self dependencies do: action].
	^count! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 11:26'!
diffBaseFor: aDependency
	| wc |
	aDependency package hasWorkingCopy ifFalse: [^nil].
	wc := aDependency package workingCopy.
	wc ancestors ifEmpty: [^nil].
	^wc ancestors first name! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 18:22'!
fileName
	^ self name, '.', self writerClass extension
! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 3/22/2005 10:51'!
fileOutOn: aStream
	self writerClass fileOut: self on: aStream! !

!MCConfiguration methodsFor: 'faking' stamp: 'bf 3/24/2005 01:17'!
info
	^MCVersionInfo new! !

!MCConfiguration methodsFor: 'initialize' stamp: 'ar 5/27/2005 17:28'!
initialize
	super initialize.
	log := DefaultLog.! !

!MCConfiguration methodsFor: 'testing' stamp: 'bf 3/22/2005 22:56'!
isCacheable
	^false! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 4/22/2005 17:19'!
load
	^self depsSatisfying: [:dep | dep isCurrent not]
		versionDo: [:ver | ver load]
		displayingProgress: 'loading packages'
! !

!MCConfiguration methodsFor: 'accessing' stamp: 'mp 2/2/2010 20:38'!
log
	"Answer the receiver's log. If no log exist use the default log"
	
	^log ifNil: [
		(name notNil and: [ self class logToFile ]) ifFalse: [ 
			Transcript countOpenTranscripts = 0 ifTrue: [Transcript open].
			^Transcript ].
		self log: ((FileStream fileNamed: self logFileName) setToEnd; yourself).
		log ]! !

!MCConfiguration methodsFor: 'accessing' stamp: 'ar 4/28/2005 11:55'!
log: aStream
	log := aStream.! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 16:07'!
logError: aString
	self log
		cr; nextPutAll: 'ERROR: ';
		nextPutAll: aString; cr;
		flush.
! !

!MCConfiguration methodsFor: 'accessing' stamp: 'ul 12/10/2009 11:56'!
logFileName

	^self name, '-', (FileDirectory localNameFor: SmalltalkImage current imageName), '.log'
	! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 15:59'!
logUpdate: aPackage with: aVersion
	self log
		cr; nextPutAll: '========== ', aVersion info name, ' =========='; cr;
		cr; nextPutAll: aVersion info message asString; cr;
		flush.

	aPackage hasWorkingCopy ifFalse: [^self].

	aPackage workingCopy ancestors do: [:each |
		(aVersion info hasAncestor: each)
			ifTrue: [(aVersion info allAncestorsOnPathTo: each)
				do: [:ver | self log cr; nextPutAll: '>>> ', ver name, ' <<<'; cr;
							nextPutAll: ver message; cr; flush]]]! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 6/9/2005 16:08'!
logWarning: aString
	self log
		cr; nextPutAll: 'WARNING: ';
		nextPutAll: aString; cr;
		flush.
! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 4/22/2005 17:19'!
merge
	^self depsSatisfying: [:dep | dep isFulfilledByAncestors not]
		versionDo: [:ver | ver merge]
		displayingProgress: 'merging packages'
! !

!MCConfiguration methodsFor: 'private' stamp: 'bf 5/23/2005 14:47'!
mustMerge: aVersion
	"answer true if we have to do a full merge and false if we can simply load instead"
	
	| pkg wc current |
	(pkg := aVersion package) hasWorkingCopy ifFalse: [^false "no wc -> load"].
	(wc := pkg workingCopy) modified ifTrue: [^true "modified -> merge"].
	wc ancestors isEmpty ifTrue: [^true "no ancestor info -> merge"].
	current := wc ancestors first.
	(aVersion info hasAncestor: current) ifTrue: [^false "direct descendant of wc -> load"].
	"new branch -> merge"
	^true! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 18:23'!
name
	^name! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 18:23'!
name: aString
	name := aString! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/23/2005 17:35'!
repositories
	^repositories ifNil: [repositories := OrderedCollection new]! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/23/2005 17:36'!
repositories: aCollection
	repositories := aCollection! !

!MCConfiguration methodsFor: 'private' stamp: 'ul 3/14/2011 17:23'!
suggestedNameOfNextVersion
	"Suggest a name for the next version of this configuration. The format is assumed to be name-authorInitials.version. Automatically increments the version, takes author initials from Utilities."

	^'{1}-{2}.{3}' format: {
		name
			ifNil: [ 'newConfiguration' ]
			ifNotNil: [ name copyUpTo: $- ].
		Utilities authorInitials.
		name
			ifNil: [ 1 ]
			ifNotNil: [ (name copyAfterLast: $.) asInteger + 1 ] }

	! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/23/2005 00:44'!
summary
	^String streamContents: [:stream |
		self dependencies
			do: [:ea | stream nextPutAll: ea versionInfo name; cr ]]! !

!MCConfiguration methodsFor: 'updating' stamp: 'bf 5/23/2005 17:43'!
updateFromImage
	self dependencies: (self dependencies collect: [:dep |
		dep package hasWorkingCopy
			ifTrue: [
				dep package workingCopy in: [:wc |
					MCVersionDependency package: wc package info: wc ancestors first]]
			ifFalse: [dep]]).
! !

!MCConfiguration methodsFor: 'updating' stamp: 'nice 10/21/2009 00:09'!
updateFromRepositories
	| oldInfos newNames sortedNames newDeps |
	oldInfos := self dependencies collect: [:dep | dep versionInfo].
	newNames := Dictionary new.
	self repositories
		do: [:repo | 
			ProgressNotification signal: '' extra: 'Checking ', repo description.
			(repo possiblyNewerVersionsOfAnyOf: oldInfos)
				do: [:newName | newNames at: newName put: repo]]
		displayingProgress: 'Searching new versions'.

	sortedNames := newNames keys asArray sort:
		[:a :b | a numericSuffix > b numericSuffix].

	newDeps := OrderedCollection new.
	self dependencies do: [:dep |
		| newName |
		newName := sortedNames
			detect: [:each | (each copyUpToLast: $-) = dep package name]
			ifNone: [nil].
		newDeps add: (newName
			ifNil: [dep]
			ifNotNil: [
				| repo info  |
				repo := newNames at: newName.
				info := self versionInfoNamed: newName for: dep from: repo.
				info ifNil: [dep]
					ifNotNil: [MCVersionDependency package: dep package info: info]
			])
	] displayingProgress: 'downloading new versions'.

	self dependencies: newDeps.
! !

!MCConfiguration methodsFor: 'actions' stamp: 'bf 5/23/2005 15:40'!
upgrade
	^self depsSatisfying: [:dep | dep isFulfilledByAncestors not]
		versionDo: [:ver | 
			(Preferences upgradeIsMerge and: [self mustMerge: ver])
				ifFalse: [ver load]
				ifTrue: [[ver merge]
					on: MCMergeResolutionRequest do: [:request |
						request merger conflicts isEmpty
							ifTrue: [request resume: true]
							ifFalse: [request pass]]]]
		displayingProgress: 'upgrading packages'
! !

!MCConfiguration methodsFor: 'private' stamp: 'ar 8/6/2009 21:48'!
versionInfoNamed: newName for: dep from: repo
	"Retrieves the version info instead of the version. Searches in-image first, in case the desired version is part of an already loaded package (usual case when doing a partial update). If not present defaults to versionNamed:for:from: an uses its result."
	MCWorkingCopy registry at: dep package ifPresent:[:workingCopy| | seen |
		"Don't use allAncestorsDo: - apparently this can loop indefinitely.
		Rather keep track of the versions that we've seen and make sure we don't loop."
		seen := Set new.
		workingCopy ancestry ancestorsDoWhileTrue:[:vInfo|
			vInfo name = newName ifTrue:[^vInfo].
			(seen includes: vInfo) ifTrue:[false] ifFalse:[seen add: vInfo. false]
		].
	].
	^(self versionNamed: newName for: dep from: repo) info! !

!MCConfiguration methodsFor: 'private' stamp: 'ar 12/30/2009 17:33'!
versionNamed: verName for: aDependency from: repo

	| baseName fileName ver |
	(repo filterFileNames: repo cachedFileNames forVersionNamed: verName) ifNotEmptyDo: [:cachedNames |
		fileName := cachedNames anyOne.
		self class extraProgressInfo
			ifTrue:[ProgressNotification signal: '' extra: 'Using cached ', fileName].
		ver := repo versionFromFileNamed: fileName].
	ver ifNil: [
		baseName := self diffBaseFor: aDependency.
		(baseName notNil and: [baseName ~= verName and: [repo includesVersionNamed: baseName]]) ifTrue: [
			fileName := (MCDiffyVersion nameForVer: verName base: baseName), '.mcd'.
			self class extraProgressInfo
				ifTrue:[ProgressNotification signal: '' extra: 'Downloading ', fileName].
			ver := repo versionFromFileNamed: fileName]].
	ver ifNil: [
		fileName := verName, '.mcz'.
		self class extraProgressInfo
			ifTrue:[ProgressNotification signal: '' extra: 'Downloading ', fileName].
		ver := repo versionFromFileNamed: fileName].
	^ver! !

!MCConfiguration methodsFor: 'accessing' stamp: 'bf 3/22/2005 10:50'!
writerClass
	^ MCMcmWriter ! !

Object subclass: #MCMcmUpdater
	instanceVariableNames: ''
	classVariableNames: 'DefaultUpdateURL LastUpdateMap SkipPackages UpdateMissingPackages'
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCMcmUpdater commentStamp: 'cbc 8/26/2010 16:42' prior: 0!
MCMcmUpdater provides utility methods for updating Monticello packages from Monticello configurations.

When Monticello configurations are stored in a repository (or repositories), MCMcmUpdater acts as an update stream. It first ensures that each configuration map has been loaded in sequence, then updates the last configuration map to the most recent version for each specified package, and finally loads these versions to produce a fully updated configuration.

Currently if a set of packages are unloaded from the image, using this class to reload them may cause problems, depending on what dependencies those classes have.  Success is not assured.  Removing packages via SmalltalkImage>>unloadAllKnownPackages will be successful, it flags the packages removed so that they are not loaded by this utility.

If you wish to not have MCMcmUpdater update packages, there are two ways to handle this:

1) To have MCMcmUpdater not update any packages not currently in the image set the UpdateMissingPackages preference to false:
		MCMcmUpdater updateMissingPackages: false
	Note that any new packages added to the repositories will not be picked up when this is turned off.
2) To have MCMcmUpdater not update a specific package, evaluate
		MCMcmUpdater disableUpdatesOfPackage: <packageName>

Class Variables definitions:

DefaultUpdateURL - String: the URL that will be checked by default for updates.  This would be set for a common standard location to check.

LastUpdateMap - Dictionary of Integer: version number of the last loaded update map per repository.  Keeps track of the last configuration map, so that the utility will not have to run through the full history in the repositories each time you ask to update.

SkipPackages - Set of Strings: names of packages to not update in MCMcmUpdater (empty by default).

UpdateMissingPackages - Boolean: if true (default), new packages in the update config map will be loaded unless they are in SkipPackages.  If false, packages not currently loaded in the image will not be loaded by MCMcmUpdater.  (This can be dangerous if packages are split - use at your own risk).
!

!MCMcmUpdater class methodsFor: 'updating' stamp: 'ar 4/9/2010 20:41'!
defaultUpdateURL
	"The default update repository URL"

	<preference: 'Update URL'
		category: 'Monticello'
		description: 'The repository URL for loading updates'
		type: #String>

	^DefaultUpdateURL ifNil:['']! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'ar 4/9/2010 20:40'!
defaultUpdateURL: aString
	"The default update repository URL"

	DefaultUpdateURL := aString! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'cbc 8/25/2010 10:27'!
disableUpdatesOfPackage: packageName
	self skipPackages add: packageName! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'cbc 8/25/2010 10:29'!
enableUpdatesForAllPackages
	SkipPackages := Set new! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'cbc 8/25/2010 10:27'!
enableUpdatesOfPackage: packageName
	self skipPackages remove: packageName ifAbsent: [].! !

!MCMcmUpdater class methodsFor: 'class initialization' stamp: 'ar 4/9/2010 20:41'!
initialize
	"MCMcmUpdater initialize"
	LastUpdateMap ifNil:[
		LastUpdateMap := Dictionary new.
	].
	DefaultUpdateURL ifNil:[
		DefaultUpdateURL := 'http://source.squeak.org/trunk'.
	].! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'cbc 8/25/2010 08:14'!
skipPackages
	^SkipPackages ifNil: [SkipPackages := Set new]! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'ar 4/9/2010 20:41'!
updateFromDefaultRepository
	"Update from the default repository only"
	^self updateFromRepositories: {self defaultUpdateURL}! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'cbc 8/25/2010 09:13'!
updateFromRepositories: repositoryUrls
	"MCMcmUpdater updateFromRepositories: #(
		'http://squeaksource.com/MCUpdateTest'
	)"

	| repos config |
	Preferences enable: #upgradeIsMerge.
	LastUpdateMap ifNil:[LastUpdateMap := Dictionary new].
	"The list of repositories to consult in order"
	repos := repositoryUrls collect:[:url| 
		MCRepositoryGroup default repositories 
			detect:[:r| r description = url]
			ifNone:[ | r |
				r := MCHttpRepository location: url user: '' password: ''.
				MCRepositoryGroup default addRepository: r.
				r]].

	"The list of updates-author.version.mcm sorted by version"
	repos do:[:r| r cacheAllFileNamesDuring:[
		| minVersion updateList allNames |
		updateList := SortedCollection new.
		minVersion := LastUpdateMap at: r description ifAbsent:[0].
		"Find all the updates-author.version.mcm files"
		'Checking ', r description
			displayProgressAt: Sensor cursorPoint
			from: 0 to: 1 during:[:bar| 
				bar value: 0.
				allNames := r allFileNames.
			].
		allNames do:[:versionedName| | version base parts author type |
			parts := versionedName findTokens: '.-'.
			parts size = 4 ifTrue:[
				base := parts at: 1.
				author := parts at: 2.
				version := [(parts at: 3) asNumber] on: Error do:[:ex| ex return: 0].
				type := parts at: 4.
			].
			(base = 'update' and:[version >= minVersion and:[type = 'mcm']]) 
				ifTrue:[updateList add: version -> versionedName]].
		
		"Proceed only if there are updates available at all."
		updateList ifNotEmpty: [
			"Now process each update file. Check if we have all dependencies and if not,
			load the entire configuration (this is mostly to skip older updates quickly)"
			updateList do:[:assoc|
				ProgressNotification signal: '' extra: 'Processing ', assoc value.
				config := r versionFromFileNamed: assoc value.
				"Skip packages that were specifically unloaded"
				config dependencies: (config dependencies 
					reject: [:dep| self skipPackages includes: dep package name]).
				self updateMissingPackages ifFalse:[
					"Skip packages that are not in the image"
					config dependencies: (config dependencies 
						select: [:dep| dep package hasWorkingCopy])].
				(config dependencies allSatisfy:[:dep| dep isFulfilled]) 
					ifFalse:[config upgrade].
				LastUpdateMap at: r description put: assoc key.
			] displayingProgress: 'Processing configurations'.
			"We've loaded all the provided update configurations.
			Use the latest configuration to update all the remaining packages."
			config updateFromRepositories.
			config upgrade.
		]].
	].
	^config! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'ar 8/24/2010 20:03'!
updateMissingPackages
	"Whether to update missing (unloaded) packages"

	<preference: 'Update missing package'
		category: 'Monticello'
		description: 'If true, missing (unloaded) packages will be loaded during the update process.'
		type: #Boolean>

	^UpdateMissingPackages ifNil:[true]! !

!MCMcmUpdater class methodsFor: 'updating' stamp: 'ar 8/24/2010 20:04'!
updateMissingPackages: aBool
	"Whether to update missing (unloaded) packages"

	UpdateMissingPackages := aBool.! !

MCVersionReader subclass: #MCMcmReader
	instanceVariableNames: 'fileName configuration'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'MonticelloConfigurations'!

!MCMcmReader commentStamp: 'dtl 5/10/2010 22:22' prior: 0!
A MCMcmReader creates an MCConfiguration by reading an array specification from a stream.

!

!MCMcmReader class methodsFor: 'accessing' stamp: 'bf 3/22/2005 10:47'!
extension
	^ 'mcm'! !

!MCMcmReader class methodsFor: 'instance creation' stamp: 'ar 9/19/2009 11:49'!
loadVersionFile: fileName
	| version |
	version := self versionFromFile: fileName.
	version load.
! !

!MCMcmReader class methodsFor: 'instance creation' stamp: 'stephaneducasse 2/4/2006 20:47'!
on: aStream fileName: aFileName
	| reader |
	reader := self on: aStream.
	reader fileName: aFileName.
	^reader! !

!MCMcmReader methodsFor: 'accessing' stamp: 'ar 6/30/2009 17:05'!
configuration
	configuration ifNil: [self loadConfiguration].
	"browser modifies configuration, but the reader might get cached"
	^configuration copy! !

!MCMcmReader methodsFor: 'accessing' stamp: 'bf 3/23/2005 01:17'!
configurationName
	^fileName ifNotNil: [(fileName findTokens: '/\:') last copyUpToLast: $.]! !

!MCMcmReader methodsFor: 'accessing' stamp: 'bf 3/23/2005 01:17'!
fileName: aString
	fileName := aString! !

!MCMcmReader methodsFor: 'accessing' stamp: 'ar 6/30/2009 17:05'!
loadConfiguration
	stream reset.
	configuration := MCConfiguration fromArray: (MCScanner scan: stream).
	configuration name: self configurationName.
! !

!MCMcmReader methodsFor: 'accessing' stamp: 'ar 6/30/2009 17:06'!
loadVersionInfo
	info := self configuration! !

!MCMcmReader methodsFor: 'accessing' stamp: 'ar 6/30/2009 17:06'!
version
	^self configuration! !
MCConfigurationBrowser initialize!
MCConfiguration initialize!
MCMcmUpdater initialize!
